/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.deploy.deployable.test.deployablefactory;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;

import java.io.File;

import org.ow2.util.archive.api.IFileArchive;
import org.ow2.util.archive.impl.ArchiveManager;
import org.ow2.util.ee.deploy.api.deployable.IDeployable;
import org.ow2.util.ee.deploy.impl.helper.DeployableHelper;
import org.ow2.util.ee.deploy.api.deployable.FileDeployable;
import org.ow2.util.ee.deploy.api.deployable.factory.IFileDeployableFactoryManager;
import org.ow2.util.ee.deploy.api.deployable.factory.XmlFileDeployableFactory;
import org.ow2.util.plan.deploy.deployable.impl.DeploymentPlanDeployableImpl;
import org.ow2.util.plan.deploy.deployable.impl.RepositoryDeployableImpl;
import org.ow2.util.ee.deploy.impl.deployable.factory.DefaultFileDeployableFactoryManager;
import org.ow2.util.ee.deploy.impl.deployable.factory.XmlFileDeployableFactoryImpl;
import org.testng.annotations.Test;

/**
 * Test class for the deployable factories.
 * @author mleduque
 */
public class FileDeployableFactoryTest {

    /**
     * Path to the test example for repositories data.
     */
    private static final String REPOSITORIES_XML_PATH =
        "target/test-classes/repositories.xml";

    /**
     * Path to the test example for deployment plan.
     */
    private static final String DEPLOYMENTPLAN_XML_PATH =
        "target/test-classes/deployment-plan.xml";

    /**
     * Deployment plan namespace.
     */
    private static final String DEPLOYMENTPLAN_NS =
    "http://jonas.ow2.org/ns/deployment-plan/1.0";

    /**
     * repositories namespace.
     */
    private static final String REPOSITORY_NS =
    "http://jonas.ow2.org/ns/deployment-plan/repositories/1.0";

    /**
     * Archive made from the deployment plan.
     */
    private IFileArchive planArchive = null;

    /**
     * Archive made from the repositories data file.
     */
    private IFileArchive repositoryArchive = null;

    /**
     * The xml deployable factory.
     */
    private XmlFileDeployableFactory factory = null;

    /**
     * The deployable factory manager.
     */
    private IFileDeployableFactoryManager manager = null;

    /**
     * Prepares test data.
     * @throws Exception
     */
    @Test
    public FileDeployableFactoryTest() throws Exception {
        // ClassCastException
        repositoryArchive = (IFileArchive) ArchiveManager.getInstance().getArchive(
                new File(REPOSITORIES_XML_PATH));
        assertNotNull(repositoryArchive);
        planArchive = (IFileArchive) ArchiveManager.getInstance().getArchive(
                new File(DEPLOYMENTPLAN_XML_PATH));
        assertNotNull(planArchive);
        factory = new XmlFileDeployableFactoryImpl();
        factory.registerFileDeployable(
                DeploymentPlanDeployableImpl.class,
                DEPLOYMENTPLAN_NS);
        factory.registerFileDeployable(
                RepositoryDeployableImpl.class,
                REPOSITORY_NS);
        manager = new DefaultFileDeployableFactoryManager();
        manager.addFileDeployableFactory(factory);
    }

    /**
     * Test "Repositories" conversion with the factory.
     * @throws Exception
     */
    @Test
    public void convertRepositoryArchiveToDeployable() throws Exception {
            FileDeployable<?, ?> fileDeployable = factory.getFileDeployable(repositoryArchive);
            assertEquals(fileDeployable.getClass(),
                    RepositoryDeployableImpl.class);
    }

    /**
     * Test deployment plan conversion with the factory.
     * @throws Exception
     */
    @Test
    public void convertDeploymentPlanArchiveToDeployable() throws Exception {
        FileDeployable<?, ?> fileDeployable = factory.getFileDeployable(planArchive);
        assertEquals(fileDeployable.getClass(),
                DeploymentPlanDeployableImpl.class);
    }

    /**
     * Test eployment plan conversion with the manager.
     * @throws Exception
     */
    @Test
    public void convertDeploymentPlanArchiveWithManager() throws Exception {
        FileDeployable<?, ?> fileDeployable = manager.getFileDeployable(planArchive);
        assertEquals(fileDeployable.getClass(),
                DeploymentPlanDeployableImpl.class);
    }

    /**
     * Test deployment plan conversion with the DeployableHelper+factory.
     * @throws Exception
     */
    @Test
    public void convertWithDeployableHelper() throws Exception {
        DeployableHelper.setFileDeployableFactory(factory);
        IDeployable<?> deployable = DeployableHelper.getDeployable(planArchive);
        assertEquals(deployable.getClass(), DeploymentPlanDeployableImpl.class);

        deployable = DeployableHelper.getDeployable(repositoryArchive);
        assertEquals(deployable.getClass(), RepositoryDeployableImpl.class);
    }

    /**
     * Test deployment plan conversion with the DeployableHelper+manager.
     * @throws Exception
     */
    @Test
    public void convertWithDeployableHelperAndManager() throws Exception {
        DeployableHelper.setFileDeployableFactory(manager);
        IDeployable<?> deployable = DeployableHelper.getDeployable(planArchive);
        assertEquals(deployable.getClass(), DeploymentPlanDeployableImpl.class);

        deployable = DeployableHelper.getDeployable(repositoryArchive);
        assertEquals(deployable.getClass(), RepositoryDeployableImpl.class);
    }

}
