/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.fetcher.impl;

import java.io.File;

import org.ow2.util.plan.bindings.deploymentplan.DeploymentPlanFragment;
import org.ow2.util.plan.bindings.exceptions.InvalidDeploymentException;
import org.ow2.util.plan.fetcher.api.IResourceFetcher;
import org.ow2.util.plan.fetcher.api.exceptions.ResourceFetcherNotResolvedException;
import org.ow2.util.plan.repository.api.IRepositoryManager;

/**
 * Abstract implementation for the resource fetchers.
 * @author Mickaël LEDUQUE
 */
public abstract class AbsResourceFetcherImpl implements IResourceFetcher {

    /**
     * The deployment.
     */
    protected DeploymentPlanFragment deployment = null;

    /**
     * The repository manager the fetcher will use.
     */
    private IRepositoryManager repositoryManager = null;

    /**
     * The local directory used to store the local copy of the resource.
     */
    protected File localRepositoriesBaseDir = null;

    /**
     * The URL read timeout.
     */
    final static protected int TIMEOUT = 3000;

    /**
     * {@inheritDoc}
     */
    public IRepositoryManager getRepositoryManager() {
        return this.repositoryManager;
    }

    /**
     * {@inheritDoc}
     */
    public void setRepositoryManager(final IRepositoryManager repositoryManager) {
        this.repositoryManager = repositoryManager;
    }

    /**
     * {@inheritDoc}
     */
    public void setDeployment(final DeploymentPlanFragment deployment) throws InvalidDeploymentException {
        if (deployment == null) {
            throw new InvalidDeploymentException("null deployment");
        }
        this.deployment = deployment;
    }

    /**
     * {@inheritDoc}
     */
    public DeploymentPlanFragment getDeployment() {
        return this.deployment;
    }

    /**
     * {@inheritDoc}
     */
    public void setLocalRepositoriesBaseDir(final File localRepositoriesBaseDir) {
        this.localRepositoriesBaseDir = localRepositoriesBaseDir;

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(this.getClass());
        if (this.deployment != null) {
            sb.append(":deployment=");
            sb.append(this.deployment);
        }
        return sb.toString();
    }

    /**
     * Returns a path to the local copy of the resource. The path is
     * <path-to-local-repositories-base-directory>/<repository-id>/<path-to-resource>
     * @return a path to the local copy of the resource.
     */
    protected static File makeLocalFilePath(final File localRepositoriesBaseDir, final String repositoryName, final String pathToResource) {

        File intermediateFile = null;
        if (repositoryName != null && !repositoryName.equals("")) {
            intermediateFile = new File(localRepositoriesBaseDir, repositoryName);
        } else {
            intermediateFile = localRepositoriesBaseDir;
        }
        return new File(intermediateFile, pathToResource);

    }

    @Override
    public boolean equals(final Object object) {
        if (!(object instanceof IResourceFetcher)) {
            return false;
        }
        IResourceFetcher other = (IResourceFetcher) object;
        try {
            return this.getResource().equals(other.getResource());
        } catch (ResourceFetcherNotResolvedException e) {
            return super.equals(object);
        }
    }
}
