/**
 * Copyright 2009-2012 Serli
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.plan.reader;

import java.io.File;
import java.io.IOException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.ValidationEventHandler;
import javax.xml.validation.Schema;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.plan.bindings.schemastream.ISchemaStreamFactory;

/**
 * Common abstract implementation for the readers.
 * @author Mickaël LEDUQUE
 */
public abstract class AbsReader implements IReader {

    /**
     * The logger.
     */
    private static final Log logger = LogFactory.getLog(AbsReader.class);

    /**
     * The unmarshaller used by this object.
     */
    protected Unmarshaller unMarshaller = null;

    /**
     * Classes that are used by concrete readers for unmarshalling.
     */
    protected Class<?>[] unmarshallClasses = null;

    /**
     * The schema that will be used when validating.
     */
    protected Schema unmarshallerSchema = null;

    /**
     * The stream factory used to build the Schema object.
     */
    protected ISchemaStreamFactory schemaStreamFactory = null;

    /**
     * Constructs an AbsReader object with a given ClassLoader object.
     * @param classLoader the classloader to use when creating a JAXB Context instance
     * @throws JAXBException when the JAXBContext initialization or
     * unmarshaller creation fails.
     */
    public AbsReader(final ClassLoader classLoader) throws Exception {
        this.initUnmarshallClasses();
        JAXBContext jaxbContext = null;
        try {
            jaxbContext = JAXBContext.newInstance(
                    getContextPathString(),classLoader
                    );
        } catch (JAXBException e) {
            logger.error("JAXB context creation error");
            throw e;
        }
        this.unMarshaller = jaxbContext.createUnmarshaller();

        //default validation event handler
        setValidationEventHandler(new ReaderValidationEventHandler(1));
    }

    /**
     * Default constructor.
     *
     * @throws JAXBException when the JAXBContext initialization or
     * unmarshaller creation fails.
     */
    public AbsReader() throws Exception {
        this(AbsReader.class.getClassLoader());
    }

    /**
     * Returns the root element unmarshalled from the given file.
     * @param file the file that will be unmarshalled.
     * @return the unmarshalled root element.
     * @throws IOException if unmarshalled failed because of IO errors.
     * @throws JAXBException if unmarshalled failed because of parsing errors.
     */
    protected JAXBElement<?> getRootElement(final File file) throws IOException, JAXBException {
        return (JAXBElement<?>)this.unMarshaller.unmarshal(file);
    }

    /**
     * Initializes the <i>unmarshallClasses</i> property.
     */
    protected abstract void initUnmarshallClasses();

    /**
     * Initializes the schema used by the reader to validate the resources.
     * @throws Exception if the creation failed.
     */
    protected abstract void buildUnmarshallerSchema() throws Exception;

    /**
     * {@inheritDoc}
     */
    public void setValidationEventHandler(final ValidationEventHandler handler) throws JAXBException {
        this.unMarshaller.setEventHandler(handler);
    }

    /**
     * {@inheritDoc}
     */
    public ValidationEventHandler getValidationEventHandler() throws JAXBException {
        return this.unMarshaller.getEventHandler();
    }

    /**
     * Create a Jaxb context path to be used to create a JAXBContext.
     * @return the context path.
     */
    private String getContextPathString() {
        if (unmarshallClasses.length == 0) {
            return "";
        }

        StringBuilder sb = new StringBuilder(unmarshallClasses[0].getPackage().getName());
        int i;
        for (i = 1; i < unmarshallClasses.length; i++) {
            sb.append(':');
            sb.append(unmarshallClasses[i].getPackage().getName());
        }
        return sb.toString();
    }

    /**
     * {@inheritDoc}
     */
    public void setValidation(final boolean validate) throws ReaderException {
        if (validate == false ){
            this.unMarshaller.setSchema(null);
        } else {
            if (this.schemaStreamFactory != null) {
                try {
                    this.buildUnmarshallerSchema();
                } catch (Exception e) {
                    throw new ReaderException(e);
                }
            }
            if (this.unmarshallerSchema != null) {
                this.unMarshaller.setSchema(this.unmarshallerSchema);
            } else {
                throw new ReaderException("Cannot validate with a null schema object");
            }

        }
    }

    /**
     * {@inheritDoc}
     */
    public boolean isValidating() {
        // Just something strange when this.unmarshallerSchema = null...
        return (this.unMarshaller.getSchema() == null);
    }

    /**
     * {@inheritDoc}
     */
    public void setSchemaStreamFactory(final ISchemaStreamFactory factory) {
        this.schemaStreamFactory = factory;
    }

    /**
     * {@inheritDoc}
     */
    public ISchemaStreamFactory getSchemaStreamFactory() {
        return this.schemaStreamFactory;
    }
}
